-------------------------------------------------------------------------------
-- ElvUI Chat Tweaks By Lockslap (US, Bleeding Hollow)
-- <Borderline Amazing>, http://ba-guild.com
-- Based on functionality provided by Prat and/or Chatter
-------------------------------------------------------------------------------
-- this module is based on popup message from prat-3.0 by the prat dev team
local Module = ElvUI_ChatTweaks:NewModule("Popup Messages", "AceConsole-3.0", "AceHook-3.0", "LibSink-2.0")
local E, _, _, P, _ = unpack(ElvUI)
local CH = E:GetModule("Chat")
local LSM = LibStub("LibSharedMedia-3.0")
local L = ElvUI_ChatTweaks.L
Module.name = L["Popup Messages"]

local keywords = {}

local db, options
local defaults = {
	profile = {
		color = {r = 1.0, g = 1.0, b = 0},	-- yellow
		output = "Popup",
		playSound = true,
		sound = "ChatTweaks Alert",
	}
}

local function UpdateKeywords(incoming)
	table.wipe(keywords)
	incoming = incoming:gsub(",%s", ",")
	
	for i = 1, #{string.split(",", incoming)} do
		local stringValue = select(i, string.split(",", incoming))
		if stringValue == "%MYNAME%" then
			stringValue = UnitName("player")
		end
		
		if stringValue ~= "" then
			keywords[stringValue] = true
		end
	end
	
end

function Module:AddMessage(frame, text, ...)
	UpdateKeywords(P["chat"]["keywords"])
	local matchedWords = {}
	for i = 1, #{string.split(" ", text)} do
		local word = select(i, string.split(" ", text))
		if not word:find("|") then
			for keyword, _ in pairs(keywords) do
				if word:lower() == keyword:lower() then
					matchedWords[word] = E.media.hexvaluecolor .. word .. '|r'
				end
			end
		end
	end
	
	if #matchedWords > 0 then
		for word, replaceWord in pairs(matchedWordws) do
			if text == word then
				text = text:gsub(word, replaceWord)
			elseif text:find(" " .. word) then
				text = text:gsub(" " .. word, " " .. replaceWord)
			elseif text:find(word .. " ") then
				text = text:gsub(word .. " ", replaceWord .. " ")
			end
		end
		
		-- send the text to libsink
		self:Pour(text, db.color.r, db.color.g, db.color.b)
		
		-- play a sound if its enabled
		if db.playSound and LSM:IsValid("sound", db.sound) then
			PlaySoundFile(LSM:Fetch("sound", db.sound), "Master")
		end
	else
		return self.hooks[frame].AddMessage(frame, text, ...)
	end
end

function Module:Decorate(frame)
	if not self:IsHooked(frame, "AddMessage") then
		self:RawHook(frame, "AddMessage", true)
	end
end

function Module:OnEnable()
	for i = 1, NUM_CHAT_WINDOWS do
		local cf = _G[("ChatFrame%d"):format(i)]
		if cf ~= COMBATLOG then
			self:RawHook(cf, "AddMessage", true)
		end
	end
	for index, frame in ipairs(self.TempChatFrames) do
		local cf = _G[frame]
		self:RawHook(cf, "AddMessage", true)
	end
end

function Module:OnDisable()
	for i = 1, NUM_CHAT_WINDOWS do
		local cf = _G[("ChatFrame%d"):format(i)]
		if cf ~= COMBATLOG and self:IsHooked(cf, "AddMessage") then
			self:Unhook(cf, "AddMessage")
		end
	end
	for index, frame in ipairs(self.TempChatFrames) do
		local cf = _G[frame]
		if self:IsHooked(cf, "AddMessage") then
			self:Unhook(cf, "AddMessage")
		end
	end
end

function Module:Popup(source, text, r, g, b, ...)
	if UIFrameIsFlashing(CT_PopupFrame) then
		UIFrameFlashRemoveFrame(CT_PopupFrame)
	end
	
	CT_PopupFrame.fadeOut = 5
	CT_PopupFrame:SetAlpha(1.0)
	CT_PopupFrameText:SetTextColor(r, g, b)
	CT_PopupFrameText:SetText(text)
	
	-- setup the font
	local font, _, style = ChatFrame1:GetFont()
	local _, fontSize = GameFontNormal:GetFont()
	CT_PopupFrameText:SetFont(font, fontSize, style)
	CT_PopupFrameText:SetNonSpaceWrap(false)
	CT_PopupFrame:SetWidth(math.min(math.max(64, CT_PopupFrameText:GetStringWidth() + 20), 520))
	CT_PopupFrame:SetHeight(64)
	CT_PopupFrame:SetBackdropBorderColor(r, g, b)
	
	CT_PopupFrameText:ClearAllPoints()
	CT_PopupFrameText:SetPoint("TOPLEFT", CT_PopupFrame, "TOPLEFT", 10, 10)
	CT_PopupFrameText:SetPoint("BOTTOMRIGHT", CT_PopupFrame, "BOTTOMRIGHT", -10, -10)
	CT_PopupFrameText:Show()
	
	-- setup the fade in/out
	local inTime, outTime, holdTime = 1, CT_PopupFrame.fadeOut, 4
	local fadeInfo, fadeInfoOut = {}, {}
	fadeInfo.timeToFade = inTime
	fadeInfo.mode = "IN"
	fadeInfo.fadeHoldTime = holdTime
	fadeInfo.startAlpha = 0
	fadeInfo.endAlpha = 1.0
	fadeInfo.finishedFunc = UIFrameFade
	fadeInfo.finishedArg1 = CT_PopupFrame
	fadeInfo.finishedArg2 = fadeInfoOut
	
	fadeInfoOut.startAlpha = 1.0
	fadeInfoOut.endAlpha = 0
	fadeInfoOut.timeToFade = outTime
	fadeInfoOut.mode = "OUT"
	fadeInfoOut.finishedFunc = function() CT_PopupFrameText:Hide() end
	
	UIFrameFade(CT_PopupFrame, fadeInfo)
end

function Module:OnInitialize()
	self.db = ElvUI_ChatTweaks.db:RegisterNamespace("PopupMessages", defaults)
	self:SetSinkStorage(self.db.profile)
	db = self.db.profile
	self.debug = ElvUI_ChatTweaks.db.profile.debugging
	UpdateKeywords(P["chat"]["keywords"])
end

function Module:GetOptions()
	if not options then
		options = {
			color = {
				type = "color",
				order = 13,
				name = L["Font Color"],
				desc = L["Set the font color."],
				get = function() return db.color.r, db.color.g, db.color.b end,
				set = function(_, r, g, b) db.color.r = r; db.color.g = g; db.color.b = b; end,
			},
			playSound = {
				type = "toggle",
				order = 14,
				name = L["Play Sound"],
				desc = L["Play a sound when it pops up."],
				get = function() return db.playSound end,
				set = function(_, value) db.playSound = value end,
			},
			sound = {
				type = "select",
				order = 15,
				dialogControl = "LSM30_Sound",
				name = L["Sound"],
				desc = L["Set the sound to play."],
				disabled = function() return not db.playSound end,
				values = AceGUIWidgetLSMlists.sound,
				get = function() return db.sound end,
				set = function(_, value) db.sound = value end,
			},
		}
		-- add our popup frame to libsink
		self:RegisterSink(L["Popup"], L["PopupMessage"], L["Shows messages in a popup frame."], "Popup")
		options.output = self:GetSinkAce3OptionsDataTable()
		options.output.order = 100
		options.output.guiInline = true
	end
	return options
end

function Module:Info()
	return L["When an ElvUI keyword is found it will show the message in a popup frame."]
end	